@@ -5,13 +5,21 @@ import java.util.Arrays;
 import java.util.Calendar;
 import java.util.Date;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
+
+import javax.mail.internet.AddressException;
+import javax.mail.internet.InternetAddress;
 
 import org.apache.commons.lang.NullArgumentException;
 import org.gcube.portal.databook.shared.Comment;
 import org.gcube.portal.databook.shared.Feed;
 import org.gcube.portal.databook.shared.FeedType;
+import org.gcube.portal.databook.shared.Invite;
+import org.gcube.portal.databook.shared.InviteOperationResult;
+import org.gcube.portal.databook.shared.InviteStatus;
 import org.gcube.portal.databook.shared.Like;
 import org.gcube.portal.databook.shared.Notification;
 import org.gcube.portal.databook.shared.NotificationChannelType;
@@ -22,6 +30,8 @@ import org.gcube.portal.databook.shared.ex.ColumnNameNotFoundException;
 import org.gcube.portal.databook.shared.ex.CommentIDNotFoundException;
 import org.gcube.portal.databook.shared.ex.FeedIDNotFoundException;
 import org.gcube.portal.databook.shared.ex.FeedTypeNotFoundException;
+import org.gcube.portal.databook.shared.ex.InviteIDNotFoundException;
+import org.gcube.portal.databook.shared.ex.InviteStatusNotFoundException;
 import org.gcube.portal.databook.shared.ex.LikeIDNotFoundException;
 import org.gcube.portal.databook.shared.ex.NotificationChannelTypeNotFoundException;
 import org.gcube.portal.databook.shared.ex.NotificationIDNotFoundException;
@@ -42,8 +52,6 @@ import com.netflix.astyanax.query.PreparedIndexExpression;
 import com.netflix.astyanax.serializers.StringSerializer;
 /**
  * @author Massimiliano Assante ISTI-CNR
- * @version 1.0 Dec 2012
- * 
  * 
  * This class is used for querying and adding data to Cassandra via Astyanax High Level API 
  *
@@ -63,6 +71,7 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 	public static final String FEEDS = "Feeds";
 	public static final String COMMENTS = "Comments";
 	public static final String LIKES = "Likes";
+	public static final String INVITES = "Invites";
 	public static final String VRE_TIMELINE_FEEDS = "VRETimeline";
 	public static final String USER_TIMELINE_FEEDS = "USERTimeline";
 	public static final String APP_TIMELINE_FEEDS = "AppTimeline";
@@ -70,6 +79,11 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 	public static final String USER_NOTIFICATIONS = "USERNotifications"; // regular user notifications timeline
 	public static final String USER_MESSAGES_NOTIFICATIONS = "USERMessagesNotifications";  // user messages notifications timeline
 	public static final String USER_NOTIFICATIONS_PREFERENCES = "USERNotificationsPreferences"; // preferences for notifications
+	public static final String HASHTAGS_COUNTER = "HashtagsCounter"; // count the hashtags per group and type
+	public static final String HASHTAGGED_FEEDS = "HashtaggedFeeds"; // contains hashtags per type associated with vre and feed
+	public static final String VRE_INVITES = "VREInvites"; //contains the emails that were invited per VRE
+	public static final String EMAIL_INVITES = "EMAILInvites"; //contains the list of invitation per email
+
 
 	private static ColumnFamily<String, String> cf_Connections = new ColumnFamily<String, String>(
 			CONNECTIONS, // Column Family Name
@@ -106,6 +120,10 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 			LIKES,     // Column Family Name
 			StringSerializer.get(),   // Key Serializer
 			StringSerializer.get());  // Column Serializer
+	private static ColumnFamily<String, String> cf_Invites = new ColumnFamily<String, String>(
+			INVITES, // Column Family Name
+			StringSerializer.get(),   // Key Serializer
+			StringSerializer.get());  // Column Serializer
 	private static ColumnFamily<String, String> cf_UserLikedFeeds = new ColumnFamily<String, String>(
 			USER_LIKED_FEEDS,     // Column Family Name
 			StringSerializer.get(),   // Key Serializer
@@ -126,6 +144,23 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 			USER_NOTIFICATIONS_PREFERENCES,  // Column Family Name
 			StringSerializer.get(),   // Key Serializer
 			StringSerializer.get());  // Column Serializer
+	private static ColumnFamily<String, String> cf_HashtagsCounter =	new ColumnFamily<String, String>(
+			HASHTAGS_COUNTER,        // Column Family Name
+			StringSerializer.get(),   // Key Serializer
+			StringSerializer.get());  // Column Serializer
+	protected static ColumnFamily<String, String> cf_HashtagTimeline =	new ColumnFamily<String, String>(
+			HASHTAGGED_FEEDS,  // Column Family Name
+			StringSerializer.get(),   // Key Serializer
+			StringSerializer.get());  // Column Serializer
+
+	private static ColumnFamily<String, String> cf_VREInvites =	new ColumnFamily<String, String>(
+			VRE_INVITES,        // Column Family Name
+			StringSerializer.get(),   // Key Serializer
+			StringSerializer.get());  // Column Serializer
+	protected static ColumnFamily<String, String> cf_EmailInvites =	new ColumnFamily<String, String>(
+			EMAIL_INVITES,  // Column Family Name
+			StringSerializer.get(),   // Key Serializer
+			StringSerializer.get());  // Column Serializer
 
 
 
@@ -133,7 +168,7 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 	 * connection instance
 	 */
 	private CassandraClusterConnection conn;
-	
+
 	protected CassandraClusterConnection getConnection() {
 		return conn;
 	}
@@ -485,6 +520,34 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 		}
 		return toReturn;
 	}
+
+	/**
+	 * helper method that return whether the user 
+	 * @param userid  user identifier
+	 * @param feedid the feed identifier
+	 * @return true if the feed id liked already
+	 */
+	private boolean isFeedLiked(String userid, String feedid) {
+		OperationResult<Rows<String, String>> result = null;
+		try {
+			result = conn.getKeyspace().prepareQuery(cf_UserLikedFeeds)
+					.getKeySlice(userid)
+					.execute();
+		} catch (ConnectionException e) {
+			e.printStackTrace();
+		}
+
+		// Iterate rows and their columns looking for the feeid id
+		for (Row<String, String> row : result.getResult()) {
+			for (Column<String> column : row.getColumns()) {
+				if (column.getStringValue().compareTo(feedid)==0)	
+					return true;
+			}
+		}
+		return false;
+	}
+
+
 	/**
 	 * helper method that retrieve all the feed Ids belonging to an application
 	 * @param appid  application identifier
@@ -737,6 +800,7 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 
 		return execute(m);
 	}
+
 	/**
 	 * {@inheritDoc}
 	 */
@@ -848,7 +912,7 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 	 * {@inheritDoc}
 	 */
 	@Override
-	public List<Notification> getAllNotificationByUser(String userid, int limit) throws NotificationTypeNotFoundException,	ColumnNameNotFoundException, NotificationIDNotFoundException {
+	public List<Notification> getAllNotificationByUser(String userid, int limit) throws NotificationTypeNotFoundException,	ColumnNameNotFoundException {
 		ArrayList<Notification> toReturn = new ArrayList<Notification>();
 		ArrayList<String> notificationsIDs = getUserNotificationsIds(userid);
 		//check if quantity is greater than user feeds
@@ -856,8 +920,13 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 
 		//need them in reverse order		
 		for (int i = notificationsIDs.size()-1; i >= (notificationsIDs.size()-limit); i--) {
-			Notification toAdd = readNotification(notificationsIDs.get(i));
-			toReturn.add(toAdd);
+			Notification toAdd = null;
+			try {
+				toAdd = readNotification(notificationsIDs.get(i));
+				toReturn.add(toAdd);
+			} catch (NotificationIDNotFoundException e) {
+				_log.error("Notification not found id=" + notificationsIDs.get(i));
+			}
 		}
 		return toReturn;	
 	}
@@ -915,18 +984,23 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 	 * {@inheritDoc}
 	 */
 	@Override
-	public boolean setAllNotificationReadByUser(String userid) throws NotificationIDNotFoundException, NotificationTypeNotFoundException, ColumnNameNotFoundException {
+	public boolean setAllNotificationReadByUser(String userid) throws NotificationTypeNotFoundException, ColumnNameNotFoundException {
 		ArrayList<String> notificationsIDs = getUserNotificationsIds(userid);
 
 		//need them in reverse order		
 		for (int i = notificationsIDs.size()-1; i >= 0; i--) {
-			Notification toAdd = readNotification(notificationsIDs.get(i));
-			if ((!toAdd.isRead()) && (toAdd.getType() != NotificationType.MESSAGE) ) {  //while I encounter unread notifications keep putting them to read, else exit
-				setNotificationRead(toAdd.getKey());
-			}
-			else {
-				break;
-			}
+			Notification toAdd;
+			try {
+				toAdd = readNotification(notificationsIDs.get(i));
+				if ((!toAdd.isRead()) && (toAdd.getType() != NotificationType.MESSAGE) ) {  //while I encounter unread notifications keep putting them to read, else exit
+					setNotificationRead(toAdd.getKey());
+				}
+				else {
+					break;
+				}
+			} catch (NotificationIDNotFoundException e) {
+				_log.error("Could not set read notification with id =" + notificationsIDs.get(i));
+			}			
 		}
 		return true;	
 	}
@@ -952,16 +1026,22 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 	 * {@inheritDoc}
 	 */
 	@Override
-	public boolean checkUnreadNotifications(String userid) throws NotificationIDNotFoundException, NotificationTypeNotFoundException, ColumnNameNotFoundException {	
+	public boolean checkUnreadNotifications(String userid) throws NotificationTypeNotFoundException, ColumnNameNotFoundException {	
 		ArrayList<String> notificationsIDs = getUserNotificationsIds(userid);
 		//since #readNotification costs time and newer notifications are iterarate first (with the reverse for below)
 		//i just see if the first non message notification (UserNotifications TimeLine) is read or not and return the value instead of iterating them one by one looking for unread()
 
 		//need them in reverse order		
 		for (int i = notificationsIDs.size()-1; i >= 0; i--) {
-			Notification toAdd = readNotification(notificationsIDs.get(i));
-			if (toAdd.getType() != NotificationType.MESSAGE)
-				return ! toAdd.isRead();
+			Notification toAdd;
+			try {
+				toAdd = readNotification(notificationsIDs.get(i));
+				if (toAdd.getType() != NotificationType.MESSAGE)
+					return ! toAdd.isRead();
+			} catch (NotificationIDNotFoundException e) {
+				_log.error("Notification not found with id = " + notificationsIDs.get(i));
+				return false;
+			}			
 		}
 		return false;
 	}
@@ -1273,25 +1353,31 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 			e.printStackTrace();
 			return false;
 		} 
-		// Inserting data
-		MutationBatch m = conn.getKeyspace().prepareMutationBatch();
-		//an entry in the feed CF
-		m.withRow(cf_Likes, like.getKey().toString())
-		.putColumn("Timestamp", like.getTime().getTime()+"", null)
-		.putColumn("Userid", like.getUserid(), null)
-		.putColumn("Feedid",like.getFeedid(), null)
-		.putColumn("FullName",like.getFullName(), null)
-		.putColumn("ThumbnailURL", like.getThumbnailURL(), null);
-		//and an entry in the UserLikesCF
-		m.withRow(cf_UserLikedFeeds, like.getUserid()).putColumn(like.getKey(), like.getFeedid(), null);
+		if (isFeedLiked(like.getUserid(), feedId)) {
+			_log.info("User " + like.getUserid() +  " already liked Feed " + feedId);
+			return true;
+		}
+		else {
+			// Inserting data
+			MutationBatch m = conn.getKeyspace().prepareMutationBatch();
+			//an entry in the feed CF
+			m.withRow(cf_Likes, like.getKey().toString())
+			.putColumn("Timestamp", like.getTime().getTime()+"", null)
+			.putColumn("Userid", like.getUserid(), null)
+			.putColumn("Feedid",like.getFeedid(), null)
+			.putColumn("FullName",like.getFullName(), null)
+			.putColumn("ThumbnailURL", like.getThumbnailURL(), null);
+			//and an entry in the UserLikesCF
+			m.withRow(cf_UserLikedFeeds, like.getUserid()).putColumn(like.getKey(), like.getFeedid(), null);
 
-		try {
-			m.execute();
-		} catch (ConnectionException e) {
-			e.printStackTrace();
-			return false;
-		}	
-		return updateFeedLikesCount(toLike, true);
+			try {
+				m.execute();
+			} catch (ConnectionException e) {
+				e.printStackTrace();
+				return false;
+			}	
+			return updateFeedLikesCount(toLike, true);
+		}
 	}
 	/**
 	 * {@inheritDoc}
@@ -1408,17 +1494,298 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 				toReturn.add(toAdd);
 			}
 		} catch (ConnectionException e) {
-			// TODO Auto-generated catch block
 			e.printStackTrace();
 		}
 		return toReturn;
 	}
+	/*
+	 * 
+	 ********************** 	HASHTAGS	***********************
+	 *
+	 */
+	/**
+	 * {@inheritDoc}
+	 */
+	@Override
+	public boolean saveHashTags(String feedid, String vreid, List<String> hashtags)	throws FeedIDNotFoundException {
+		Set<String> noduplicatesHashtags = null;
+		if (hashtags != null && !hashtags.isEmpty()) {
+			noduplicatesHashtags = new HashSet<String>(hashtags);
+		}
+		// Inserting data
+		MutationBatch m = conn.getKeyspace().prepareMutationBatch();
+		for (String hashtag : noduplicatesHashtags) {
+			String lowerCaseHashtag = hashtag.toLowerCase();
+			m.withRow(cf_HashtagTimeline, lowerCaseHashtag).putColumn(feedid, vreid, null);
+			boolean firstInsert = execute(m);
+			boolean secondInsert = updateVREHashtagCount(vreid, lowerCaseHashtag, true);
+			if (! (firstInsert && secondInsert)) {
+				_log.error("saveHashTags: Could not save the hashtag(s)");
+				return false;
+			}
+		}
+		return true;
+	}
+	/**
+	 * {@inheritDoc}
+	 */
+	@Override
+	public boolean deleteHashTags(String feedid, String vreid, List<String> hashtags) throws FeedIDNotFoundException {
+		Set<String> noduplicatesHashtags = null;
+		if (hashtags != null && !hashtags.isEmpty()) {
+			noduplicatesHashtags = new HashSet<String>(hashtags);
+		}
+		// Inserting data
+		MutationBatch m = conn.getKeyspace().prepareMutationBatch();
+		for (String hashtag : noduplicatesHashtags) {
+			String lowerCaseHashtag = hashtag.toLowerCase();
+			m.withRow(cf_HashtagTimeline, lowerCaseHashtag).deleteColumn(feedid);
+			boolean firstDelete = execute(m);
+			boolean secondInsert = updateVREHashtagCount(vreid, lowerCaseHashtag, false);
+			if (! (firstDelete && secondInsert)) {
+				_log.error("deleteHashTags: Could not delete the hashtag(s)");
+				return false;
+			}
+		}
+		return true;
+	}
+	/**
+	 * {@inheritDoc}
+	 */
+	@Override
+	public Map<String, Integer> getVREHashtagsWithOccurrence(String vreid) {
+		OperationResult<Rows<String, String>> result = null;
+		try {
+			result = conn.getKeyspace().prepareQuery(cf_HashtagsCounter)
+					.getKeySlice(vreid)
+					.execute();
+		} catch (ConnectionException e) {
+			e.printStackTrace();
+		}
+
+		HashMap<String, Integer> toReturn = new HashMap<String, Integer> ();
+
+		// Iterate rows and their columns 
+		for (Row<String, String> row : result.getResult()) {
+			for (Column<String> column : row.getColumns()) {
+				int curValue = Integer.parseInt(column.getStringValue());
+				if (curValue > 0)
+					toReturn.put(column.getName(), curValue);
+			}
+		}
+		return toReturn;
+	}
+	/**
+	 * {@inheritDoc}
+	 */
+	@Override
+	public List<Feed> getVREFeedsByHashtag(String vreid, String hashtag) throws PrivacyLevelTypeNotFoundException, FeedTypeNotFoundException, FeedIDNotFoundException, ColumnNameNotFoundException {
+		List<Feed> toReturn = new ArrayList<Feed>();
+		OperationResult<Rows<String, String>> result = null;
+		try {
+			result = conn.getKeyspace().prepareQuery(cf_HashtagTimeline)
+					.getKeySlice(hashtag)
+					.execute();
+		} catch (ConnectionException e) {
+			e.printStackTrace();
+		}		
+		ArrayList<String> feedIds = new ArrayList<String>();
+		// Iterate rows and their columns 
+		for (Row<String, String> row : result.getResult()) {
+			for (Column<String> column : row.getColumns()) {
+				if (column.getStringValue().compareTo(vreid)==0)
+					feedIds.add(column.getName());
+			}
+		}
+		toReturn = getFeedsByIds(feedIds);
+		return toReturn;
+	}
+	/*
+	 * 
+	 ********************** 	Invites	***********************
+	 *
+	 */
+	/**
+	 * common part to save a feed
+	 * @param feed
+	 * @return the partial mutation batch instance
+	 */
+	private MutationBatch initSaveInvite(Invite invite) {
+		if (invite == null)
+			throw new NullArgumentException("Invite instance is null");
+		// Inserting data
+		MutationBatch m = conn.getKeyspace().prepareMutationBatch();
+		//an entry in the feed CF
+		m.withRow(cf_Invites, invite.getKey().toString())
+		.putColumn("SenderUserId", invite.getSenderUserId(), null)
+		.putColumn("Vreid", invite.getVreid(), null)
+		.putColumn("InvitedEmail", invite.getInvitedEmail(), null)
+		.putColumn("ControlCode", invite.getControlCode(), null)
+		.putColumn("Status", invite.getStatus().toString(), null)
+		.putColumn("Time", invite.getTime().getTime()+"", null)
+		.putColumn("SenderFullName", invite.getSenderFullName(), null);
+		return m;
+	}
+
+	/**
+	 * {@inheritDoc}
+	 */
+	@Override
+	public String isExistingInvite(String vreid, String email) {
+		OperationResult<Rows<String, String>> result = null;
+		try {
+			result = conn.getKeyspace().prepareQuery(cf_EmailInvites)
+					.getKeySlice(email)
+					.execute();
+		} catch (ConnectionException e) {
+			e.printStackTrace();
+		}		
+		// Iterate rows and their columns 
+		for (Row<String, String> row : result.getResult()) {
+			for (Column<String> column : row.getColumns()) {
+				if (column.getName().compareTo(vreid)==0)
+					return column.getStringValue();
+			}
+		}
+		return null;
+	}
+	/**
+	 * {@inheritDoc}
+	 */
+	@Override
+	public InviteOperationResult saveInvite(Invite invite) throws AddressException {
+		if (invite == null)
+			throw new NullArgumentException("Invite instance is null");
+		String email = invite.getInvitedEmail();
+		if (! verifyEmail(email))
+			throw new AddressException("Email is not valid ->" + email);
+		if (invite.getVreid() == null || invite.getVreid().equals(""))
+			throw new NullArgumentException("VREId is null or empty");
+		_log.debug("isExistingInvite? " + invite.getInvitedEmail() + " in " + invite.getVreid());
+		if (isExistingInvite(invite.getVreid(), invite.getInvitedEmail()) != null)
+			return InviteOperationResult.ALREADY_INVITED;
+		_log.debug("Invite not found, proceed to save it ...");
+
+		MutationBatch m = initSaveInvite(invite);
+		//an entry in the VRE Invites 
+		m.withRow(cf_VREInvites, invite.getVreid())
+		.putColumn(invite.getKey().toString(), InviteStatus.PENDING.toString(), null);
+
+		//an entry in the EMAIL Invites 
+		m.withRow(cf_EmailInvites, email)
+		.putColumn(invite.getVreid(), invite.getKey().toString(), null);
+		boolean result = execute(m);
+		return result ? InviteOperationResult.SUCCESS : InviteOperationResult.FAILED;
+	}
+	/**
+	 * {@inheritDoc}
+	 */
+	@Override
+	public Invite readInvite(String inviteid) throws InviteIDNotFoundException, InviteStatusNotFoundException {
+		Invite toReturn = new Invite();
+		OperationResult<ColumnList<String>> result;
+		try {
+			result = conn.getKeyspace().prepareQuery(cf_Invites)
+					.getKey(inviteid)
+					.execute();
+
+			ColumnList<String> columns = result.getResult();
+			if (columns.size() == 0) {
+				throw new InviteStatusNotFoundException("The requested inviteid: " + inviteid + " is not existing");
+			}
+
+			toReturn.setKey(inviteid);
+			toReturn.setSenderUserId(columns.getColumnByName("SenderUserId").getStringValue());
+			toReturn.setVreid(columns.getColumnByName("Vreid").getStringValue());
+			toReturn.setInvitedEmail(columns.getColumnByName("InvitedEmail").getStringValue());
+			toReturn.setControlCode(columns.getColumnByName("ControlCode").getStringValue());
+			InviteStatus status = getInviteStatusType(columns.getColumnByName("Status").getStringValue());
+			toReturn.setStatus(status);
+			toReturn.setTime(getDateFromTimeInMillis(columns.getColumnByName("Time").getStringValue()));
+			toReturn.setSenderFullName(columns.getColumnByName("SenderFullName").getStringValue());
+
+		} catch (ConnectionException e) {
+			e.printStackTrace();
+			return null;
+		}	
+		return toReturn;
+	}
+	/**
+	 * helper method that retrieve all the Invites belonging to a list of Ids
+	 * @param inviteIds the lisf of invites UUID
+	 * @return all the invites belonging to a list of Ids
+	 * @throws InviteIDNotFoundException 
+	 * @throws InviteStatusNotFoundException 
+	 */
+	private List<Invite> getInvitesById(List<String> inviteIds) throws InviteIDNotFoundException, InviteStatusNotFoundException {
+		ArrayList<Invite> toReturn = new ArrayList<Invite>();
+		for (String inviteid : inviteIds)  
+			toReturn.add(readInvite(inviteid));
+
+		return toReturn;
+	}
+	/**
+	 * {@inheritDoc}
+	 * @throws InviteStatusNotFoundException 
+	 */
+	@Override
+	public boolean setInviteStatus(String vreid, String email, InviteStatus status) throws InviteIDNotFoundException, InviteStatusNotFoundException {
+		String inviteid = isExistingInvite(vreid, email);
+		Invite toSet = readInvite(inviteid);
+		if (toSet == null)
+			throw new InviteIDNotFoundException("The specified invite to set with id: " + inviteid + " does not exist");
+
+		MutationBatch m = conn.getKeyspace().prepareMutationBatch();
+		//update in the Invites Static CF
+		m.withRow(cf_Invites, inviteid).putColumn("Status", status.toString(), null);
+		//updated in the VREInvites Dynamic CF
+		m.withRow(cf_VREInvites, toSet.getVreid()).putColumn(inviteid, status.toString(), null);
+		try {
+			m.execute();
+		} catch (ConnectionException e) {
+			_log.error("ERROR while setting Invite " + inviteid + " to " + status.toString());
+			return false;
+		}
+		_log.trace("Invite Status Set to " +  status.toString() + " OK");
+		return true;
+	}
+	/**
+	 * {@inheritDoc}
+	 */
+	@Override
+	public List<Invite> getInvitedEmailsByVRE(String vreid, InviteStatus... status) throws InviteIDNotFoundException, InviteStatusNotFoundException{
+		OperationResult<Rows<String, String>> result = null;
+		try {
+			result = conn.getKeyspace().prepareQuery(cf_VREInvites)
+					.getKeySlice(vreid)
+					.execute();
+		} catch (ConnectionException e) {
+			e.printStackTrace();
+		}		
+		ArrayList<String> invitesIds = new ArrayList<String>();
+		// Iterate rows and their columns 
+		for (Row<String, String> row : result.getResult()) {
+			if (status != null) {
+				for (Column<String> column : row.getColumns()) {
+					for (int i = 0; i < status.length; i++) {
+						if (column.getStringValue().compareTo(status[i].toString())==0)
+							invitesIds.add(column.getName());
+					}
+
+				}
+			}
+			else {
+				for (Column<String> column : row.getColumns()) 
+					invitesIds.add(column.getName());
+			}
+		}
+		return getInvitesById(invitesIds);
+	}
 	/**
 	 * {@inheritDoc}
 	 */
 	@Override
 	public void closeConnection() {
-		// TODO Auto-generated method stub
 	}
 	/*
 	 * 
@@ -1495,6 +1862,28 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 			throw new FeedTypeNotFoundException("The Feed Type was not recognized should be one of " + FeedType.values() + " asked for: " + type);
 	}
 
+	/**
+	 * simply return an enum representing the invite status type
+	 * @param type .
+	 * @return correct enum representing the feed type
+	 * @throws TypeNotFoundException .
+	 */
+	private InviteStatus getInviteStatusType(String type) throws InviteStatusNotFoundException {
+		switch (type) {
+		case "PENDING":
+			return InviteStatus.PENDING;
+		case "ACCEPTED":
+			return InviteStatus.ACCEPTED;
+		case "REJECTED":
+			return InviteStatus.REJECTED;
+		case "RETRACTED":
+			return InviteStatus.RETRACTED;
+		default:
+			throw new InviteStatusNotFoundException("The Invite Status was not recognized should be one of " + InviteStatus.values() + " asked for: " + type);
+		}
+
+	}
+
 	/**
 	 * simply return an enum representing the feed type
 	 * @param type .
@@ -1508,6 +1897,12 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 		else if (type.compareTo("WP_FOLDER_UNSHARE") == 0) {
 			return NotificationType.WP_FOLDER_UNSHARE;
 		}
+		else if (type.compareTo("WP_ADMIN_UPGRADE") == 0) {
+			return NotificationType.WP_ADMIN_UPGRADE;
+		}
+		else if (type.compareTo("WP_ADMIN_DOWNGRADE") == 0) {
+			return NotificationType.WP_ADMIN_DOWNGRADE;
+		}
 		else if (type.compareTo("WP_FOLDER_RENAMED") == 0) {
 			return NotificationType.WP_FOLDER_RENAMED;
 		}
@@ -1594,7 +1989,16 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 		}
 		else if (type.compareTo("DOCUMENT_WORKFLOW_FIRST_STEP_REQUEST_INVOLVMENT") == 0) {
 			return NotificationType.DOCUMENT_WORKFLOW_FIRST_STEP_REQUEST_INVOLVMENT;
-		}		
+		}	
+		else if (type.compareTo("TDM_TAB_RESOURCE_SHARE") == 0) {
+			return NotificationType.TDM_TAB_RESOURCE_SHARE;
+		}	
+		else if (type.compareTo("TDM_RULE_SHARE") == 0) {
+			return NotificationType.TDM_RULE_SHARE;
+		}	
+		else if (type.compareTo("TDM_TEMPLATE_SHARE") == 0) {
+			return NotificationType.TDM_TEMPLATE_SHARE;
+		}	
 		else if (type.compareTo("GENERIC") == 0) {
 			return NotificationType.GENERIC;
 		}
@@ -1669,6 +2073,60 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 		return true;
 	}
 
+	/**
+	 * update the hashtag count by incrementing or decrementing it by (1) 
+	 * used when adding or removing a hashtag in a feed
+	 * @param vreid the vreid
+	 * @param hashtag the hashtag
+	 * @param increment set true if you want to add 1, false to subtract 1.
+	 */
+	private boolean updateVREHashtagCount(String vreid, String hashtag, boolean increment) {
+		Map<String, Integer> vreHashtags = getVREHashtagsWithOccurrence(vreid);
+		//if the hashtag not yet exist 
+		int newCount = 0;
+
+		if (!vreHashtags.containsKey(hashtag)) {
+			newCount = 1;
+		}
+		else {
+			try { 
+				int current = vreHashtags.get(hashtag);
+				newCount = increment ? current+1 : current-1;
+			}
+			catch (NumberFormatException e) {
+				_log.error("Hashtag Number found is not a number: " + newCount);
+			}
+		}
+		_log.debug("Updating counter for " + hashtag + " to " + newCount);
+		MutationBatch m = conn.getKeyspace().prepareMutationBatch();
+		m.withRow(cf_HashtagsCounter, vreid).putColumn(hashtag, ""+newCount, null);
+		try {
+			m.execute();
+		} catch (ConnectionException e) {
+			_log.error("Hashtag Count update NOT OK ");
+			return false;
+		}
+		_log.debug("Hashtag Count update OK to: " + newCount);
+		return true;
+	}
+	/**
+	 * verify an email address
+	 * @param email
+	 * @return true or false
+	 */
+	private boolean verifyEmail(String email) {
+		boolean isValid = false;
+		try {
+			InternetAddress internetAddress = new InternetAddress(email);
+			internetAddress.validate();
+			isValid = true;
+		} catch (AddressException e) {
+			_log.error("Validation Exception Occurred for email: " + email);
+		}
+		return isValid;
+	}
+
+
 
 
 
