@@ -1,37 +1,16 @@
 package org.gcube.portal.databook.server;
 
-import static org.junit.Assert.*;
-
-import java.sql.Savepoint;
-import java.util.ArrayList;
 import java.util.Date;
-import java.util.LinkedList;
 import java.util.List;
-import java.util.Map;
 import java.util.UUID;
 
-import org.gcube.portal.databook.shared.Feed;
-import org.gcube.portal.databook.shared.FeedType;
-import org.gcube.portal.databook.shared.NotificationChannelType;
-import org.gcube.portal.databook.shared.NotificationType;
-import org.gcube.portal.databook.shared.PrivacyLevel;
-import org.gcube.portal.databook.shared.ex.FeedIDNotFoundException;
-import org.gcube.portal.databook.shared.ex.NotificationChannelTypeNotFoundException;
-import org.gcube.portal.databook.shared.ex.NotificationTypeNotFoundException;
+import org.gcube.portal.databook.shared.Invite;
+import org.gcube.portal.databook.shared.InviteOperationResult;
+import org.gcube.portal.databook.shared.InviteStatus;
 import org.junit.AfterClass;
 import org.junit.BeforeClass;
 import org.junit.Test;
 
-import com.google.common.collect.ImmutableMap;
-import com.netflix.astyanax.connectionpool.OperationResult;
-import com.netflix.astyanax.connectionpool.exceptions.ConnectionException;
-import com.netflix.astyanax.ddl.ColumnFamilyDefinition;
-import com.netflix.astyanax.model.ColumnFamily;
-import com.netflix.astyanax.model.Row;
-import com.netflix.astyanax.model.Rows;
-import com.netflix.astyanax.serializers.StringSerializer;
-import com.netflix.astyanax.thrift.ddl.ThriftColumnFamilyDefinitionImpl;
-
 public class DatabookCassandraTest {
 	private static DBCassandraAstyanaxImpl store;
 
@@ -45,74 +24,125 @@ public class DatabookCassandraTest {
 		store.closeConnection();
 		System.out.println("End");
 	}
-	
-	
+
 	@Test
-	public void testHashTag() {
-		try {	
-			final String VREID = "/gcube/devsec/devVRE";
-			final String HASHTAG1 = "#testHashTag";
-			final String HASHTAG2 = "#testHashTag3";
-			List<String> hashtags = new LinkedList<String>();
-			hashtags.add(HASHTAG1);
-			hashtags.add(HASHTAG2);
-			
-//			Feed feed = new Feed(UUID.randomUUID().toString(), FeedType.TWEET, "massimiliano.assante", new Date(), VREID, 
-//					"www.d4science.org/monitor", "thumbUri", "This is a feed with " + HASHTAG1 + " and " + HASHTAG2, PrivacyLevel.VRES, "Massimiliano Assante", "massimiliano.assante@isti.cnr.it", "thumburl", "linkTitle", "linkDesc", "host");
-//			assertTrue(store.saveUserFeed(feed));
-//			assertTrue(store.saveHashTags(feed.getKey(), VREID, hashtags));
-//			assertTrue(store.deleteHashTags("d0c64e42-9616-4e24-a65a-7a63a280d676", VREID, hashtags));
-//			System.out.println(feed);
-//			
-			System.out.println("\ngetting getVREHashtagsWithOccurrence for " + VREID);
-			Map<String, Integer> hashtagsWithOcc = store.getVREHashtagsWithOccurrence(VREID);
-			for (String hashtag : hashtagsWithOcc.keySet()) {
-				System.out.println(hashtag + ":" + hashtagsWithOcc.get(hashtag));
+	public void testInvites()  {
+		String controlCode = UUID.randomUUID().toString();
+		String vreid = "/gcube/devsec/devVRE";
+		String email = "ciccio@gmail.com";
+		
+		InviteStatus statusToLookfor = InviteStatus.ACCEPTED;
+		
+		Invite invite = new Invite(UUID.randomUUID().toString(), "andrea.rossi", vreid, email, controlCode, InviteStatus.PENDING, new Date(), "Andrea Rossi");
+		try {
+			InviteOperationResult result = store.saveInvite(invite);
+			System.out.println(result);
+			String inviteid = store.isExistingInvite(vreid, email);
+//			System.out.println(store.readInvite(inviteid));
+//			store.setInviteStatus(vreid, email, InviteStatus.ACCEPTED);
+//			System.out.println(store.readInvite(inviteid));
+			System.out.println("Looking for all invites in " + vreid + " with status " + statusToLookfor);
+			List<Invite> invites = store.getInvitedEmailsByVRE(vreid, statusToLookfor, InviteStatus.PENDING);
+			for (Invite invite2 : invites) {
+				System.out.println(invite2);
 			}
-			
-			System.out.println("\ngetting getVREFeedsByHashtag for " + VREID + " and " + HASHTAG1);
-			for (Feed theFeed : store.getVREFeedsByHashtag(VREID, HASHTAG1)) {
-				System.out.println(theFeed);
-			}
-			
 		} catch (Exception e) {
 			e.printStackTrace();
 		}
-		
-		
+
 	}
-//	/**
-//	 * use exclusively to add a new (Dynamic) CF to a keyspace
-//	 */
-//	@Test
-//	public void addHashtagsColumnFamilies() {
-//		ColumnFamily<String, String> cf_HashtagsCounter =	new ColumnFamily<String, String>(
-//				DBCassandraAstyanaxImpl.HASHTAGS_COUNTER,        // Column Family Name
-//				StringSerializer.get(),   // Key Serializer
-//				StringSerializer.get());  // Column Serializer
-//		ColumnFamily<String, String> cf_HashtagTimeline =	new ColumnFamily<String, String>(
-//				DBCassandraAstyanaxImpl.HASHTAGGED_FEEDS,  // Column Family Name
-//				StringSerializer.get(),   // Key Serializer
-//				StringSerializer.get());  // Column Serializer
-//
-//		try {
-//			new CassandraClusterConnection(false).getKeyspace().createColumnFamily(cf_HashtagsCounter, ImmutableMap.<String, Object>builder()
-//					.put("default_validation_class", "UTF8Type")
-//					.put("key_validation_class",     "UTF8Type")
-//					.put("comparator_type",          "UTF8Type")
-//					.build());
-//			
-//			new CassandraClusterConnection(false).getKeyspace().createColumnFamily(cf_HashtagTimeline, ImmutableMap.<String, Object>builder()
-//					.put("default_validation_class", "UTF8Type")
-//					.put("key_validation_class",     "UTF8Type")
-//					.put("comparator_type",          "UTF8Type")
-//					.build());
-//			
-//		} catch (ConnectionException e) {
-//			e.printStackTrace();
-//		}
-//	}
-	
+
+	//	@Test
+	//	public void testHashTag() {
+	//		try {	
+	//			final String VREID = "/gcube/devsec/devVRE";
+	//			final String HASHTAG1 = "#testHashTag";
+	//			final String HASHTAG2 = "#testHashTag3";
+	//			List<String> hashtags = new LinkedList<String>();
+	//			hashtags.add(HASHTAG1);
+	//			hashtags.add(HASHTAG2);
+	//			
+	////			Feed feed = new Feed(UUID.randomUUID().toString(), FeedType.TWEET, "massimiliano.assante", new Date(), VREID, 
+	////					"www.d4science.org/monitor", "thumbUri", "This is a feed with " + HASHTAG1 + " and " + HASHTAG2, PrivacyLevel.VRES, "Massimiliano Assante", "massimiliano.assante@isti.cnr.it", "thumburl", "linkTitle", "linkDesc", "host");
+	////			assertTrue(store.saveUserFeed(feed));
+	////			assertTrue(store.saveHashTags(feed.getKey(), VREID, hashtags));
+	////			assertTrue(store.deleteHashTags("d0c64e42-9616-4e24-a65a-7a63a280d676", VREID, hashtags));
+	////			System.out.println(feed);
+	////			
+	//			System.out.println("\ngetting getVREHashtagsWithOccurrence for " + VREID);
+	//			Map<String, Integer> hashtagsWithOcc = store.getVREHashtagsWithOccurrence(VREID);
+	//			for (String hashtag : hashtagsWithOcc.keySet()) {
+	//				System.out.println(hashtag + ":" + hashtagsWithOcc.get(hashtag));
+	//			}
+	//			
+	//			System.out.println("\ngetting getVREFeedsByHashtag for " + VREID + " and " + HASHTAG1);
+	//			for (Feed theFeed : store.getVREFeedsByHashtag(VREID, HASHTAG1)) {
+	//				System.out.println(theFeed);
+	//			}
+	//			
+	//		} catch (Exception e) {
+	//			e.printStackTrace();
+	//		}
+	//		
+	//		
+	//	}
+
+
+	//	/**
+	//	 * use exclusively to add a new (Static) CF to a keyspace
+	//	 */
+	//	@Test
+	//	public void addInvitesStaticColumnFamilies() {
+	//		ColumnFamily<String, String> CF_INVITES = ColumnFamily.newColumnFamily(DBCassandraAstyanaxImpl.INVITES, StringSerializer.get(), StringSerializer.get());
+	//
+	//		try {
+	//			new CassandraClusterConnection(false).getKeyspace().createColumnFamily(CF_INVITES, ImmutableMap.<String, Object>builder()
+	//			        .put("default_validation_class", "UTF8Type")
+	//			        .put("key_validation_class",     "UTF8Type")
+	//			        .put("comparator_type",          "UTF8Type")
+	//			        .build());
+	//
+	//		} catch (ConnectionException e) {
+	//			e.printStackTrace();
+	//		}
+	//		System.out.println("addInvitesStaticColumnFamily END");
+	//	}
+
+
+	//	/**
+	//	 * use exclusively to add a new (Dynamic) CF to a keyspace
+	//	 */
+	//	@Test
+	//	public void addInvitesDynamicColumnFamilies() {
+	//		System.out.println("addInvitesColumnFamilies");
+	//		ColumnFamily<String, String> cf_HashtagsCounter =	new ColumnFamily<String, String>(
+	//				DBCassandraAstyanaxImpl.VRE_INVITES,        // Column Family Name
+	//				StringSerializer.get(),   // Key Serializer
+	//				StringSerializer.get());  // Column Serializer
+	//		ColumnFamily<String, String> cf_HashtagTimeline =	new ColumnFamily<String, String>(
+	//				DBCassandraAstyanaxImpl.EMAIL_INVITES,  // Column Family Name
+	//				StringSerializer.get(),   // Key Serializer
+	//				StringSerializer.get());  // Column Serializer
+	//
+	//		try {
+	//			new CassandraClusterConnection(false).getKeyspace().createColumnFamily(cf_HashtagsCounter, ImmutableMap.<String, Object>builder()
+	//					.put("default_validation_class", "UTF8Type")
+	//					.put("key_validation_class",     "UTF8Type")
+	//					.put("comparator_type",          "UTF8Type")
+	//					.build());
+	//			
+	//			new CassandraClusterConnection(false).getKeyspace().createColumnFamily(cf_HashtagTimeline, ImmutableMap.<String, Object>builder()
+	//					.put("default_validation_class", "UTF8Type")
+	//					.put("key_validation_class",     "UTF8Type")
+	//					.put("comparator_type",          "UTF8Type")
+	//					.build());
+	//			
+	//		} catch (ConnectionException e) {
+	//			e.printStackTrace();
+	//		}
+	//		System.out.println("addInvitesColumnFamilies END");
+	//	}
+
 
 	//	private List<String> getKeys() {
 	//		List<String> toReturn = new ArrayList<String>();
